@@ -1,7 +1,7 @@
 #!/usr/bin/perl -w
 ############################################################
 #
-#   $Id$
+#   $Id: example.pl 941 2007-02-06 18:48:19Z nicolaw $
 #   example.pl - WWW::FleXtel example script
 #
 #   Copyright 2007 Nicola Worthington
@@ -28,20 +28,39 @@ use WWW::FleXtel qw();
 use Data::Dumper qw(Dumper);
 
 my %acct = (
-		account  => 'A999999',
-		password => 'password',
-		number   => '07010000000',
-		pin      => '1234',
+		#account   => 'A999999',
+		#password  => 'password',
+		number    => '07010000000',
+		pin       => '1234',
+		cache_ttl => 15,
+		timeout   => 15,
 	);
 
 my $flextel = WWW::FleXtel->new(%acct);
 
-printf("Diverted to %s\n", $flextel->get_destination);
 my $destination = $flextel->set_destination(destination => "0800883322");
-printf("Diverted to %s\n", $destination);
-    
+
+printf("Email: %s\nDestination: %s\nICD: %s\n",
+		$flextel->get_email,
+		$flextel->get_destination,
+		$flextel->get_icd,
+	);
+
+print "Sleeping 10 seconds ...\n";
+sleep 10;
+
 print Dumper($flextel->get_phonebook);
 
+print "Sleeping 10 seconds ...\n";
+sleep 10;
+
+printf("Email: %s\nDestination: %s\nICD: %s\n",
+		$flextel->get_email,
+		$flextel->get_destination,
+		$flextel->get_icd,
+	);
+
+
 exit;
 
 __END__
