@@ -1,6 +1,6 @@
 ############################################################
 #
-#   $Id$
+#   $Id: FleXtel.pm 942 2007-02-06 18:51:21Z nicolaw $
 #   WWW::FleXtel - Manipulate FleXtel phone number redirection
 #
 #   Copyright 2007 Nicola Worthington
@@ -30,7 +30,7 @@ use Scalar::Util qw(refaddr);
 use Carp qw(croak cluck carp confess);
 use vars qw($VERSION $DEBUG);
 
-$VERSION = '0.02' || sprintf('%d', q$Revision: 809 $ =~ /(\d+)/g);
+$VERSION = '0.03' || sprintf('%d', q$Revision: 942 $ =~ /(\d+)/g);
 $DEBUG ||= $ENV{DEBUG} ? 1 : 0;
 
 my $objstore = {};
@@ -50,7 +50,7 @@ sub new {
 	my $stor = $objstore->{refaddr($self)};
 
 	# Define what parameters are valid for this constructor
-	$stor->{validkeys} = [qw(password account pin number)];
+	$stor->{validkeys} = [qw(password account pin number timeout cache_ttl)];
 	my $validkeys = join('|',@{$stor->{validkeys}});
 
 	# Only accept sensible known parameters from punters
@@ -60,7 +60,9 @@ sub new {
 		if @invalidkeys && $^W;
 
 	# Set some default values
+	delete $stor->{timeout} if !defined $stor->{timeout} || $stor->{timeout} !~ /^[1-9]\d*$/;
 	$stor->{timeout} ||= 15; # 15 seconds
+	delete $stor->{cache_ttl} if !defined $stor->{cache_ttl} || $stor->{cache_ttl} !~ /^\d+$/;
 	$stor->{cache_ttl} ||= 5; # Cache data for 5 seconds
 	$stor->{'user-agent'} ||= sprintf('Mozilla/5.0 (X11; U; Linux i686; '.
 				'en-US; rv:1.8.1.1) Gecko/20060601 Firefox/2.0.0.1 (%s %s)',
@@ -84,6 +86,7 @@ sub set_destination { &_executeQuery; }
 sub get_destination { &_executeQuery; }
 sub get_phonebook   { &_executeQuery; }
 sub get_email       { &_executeQuery; }
+sub get_icd         { &_executeQuery; }
 
 
 
@@ -230,11 +233,38 @@ sub _extractData {
 	my $html = shift;
 	my %data;
 
-	if ($html =~ /^\s*([0-9\#\+]{8,})\s*$/s) {
-		$data{destination} = $1;
+	if ($html =~ /^\s*([0-9\#\+]{8,}(?:,.*)?)\s*$/s) {
+		my @args = split(/\s*,\s*/,$1);
+		s/(^\s*|\s*$)//gs for @args;
+		DUMP('@args',\@args);
+		# 01923000009,,01923111119,01992222221,01933333368,,,,,,nicolaw@lilacup.2x4b.com
+		# destination
+		# ICD destination
+		# memory 1, memory 2, memory 3, memory 4, memory 5, memory 6, memory 7, memory 8
+		# email address (flextel number specific - not account holder email)
+		# label 1, label 2, label 3, label 4, label 5, label 6, label 7, label 8
+
+		$data{destination} = shift @args;
+		if (@args) {
+			$data{icd} = shift @args;
+			for (1..8) {
+				my $mem = shift @args;
+				$data{phonebook}->[$_]->{number} = $mem;
+				$data{phonebook}->[$_]->{memory} = $_;
+				TRACE("memory $_ => '$mem'");
+			}
+			$data{email} = shift @args;
+			for (1..8) {
+				my $title = shift @args;
+				$data{phonebook}->[$_]->{title} = $title;
+				$data{phonebook}->[$_]->{memory} = $_;
+				TRACE("memory title $_ => '$title'");
+			}
+		}
 		return \%data;
 	}
 
+	# Nasty Javascript scraping
 	for (split(/[\n\r]/,$html)) {
 		chomp;
 		if (my ($key,$num,$val) = $_ =~
@@ -242,12 +272,14 @@ sub _extractData {
 			$val =~ s/^\s*"\s*//g;
 			$val =~ s/\s*"\s*$//g;
 
-			if ($key =~ /^mem\d+$/) {
+			if (my ($index) = $key =~ /^mem(\d+)$/) {
 				$val =~ s/[^0-9\#]//g;
 				$data{phonebook}->[$num]->{number} = $val;
+				$data{phonebook}->[$num]->{memory} = $index;
 
-			} elsif ($key =~ /^mem\d+text$/) {
+			} elsif ($key =~ /^mem(\d+)text$/) {
 				$data{phonebook}->[$num]->{title} = $val;
+				$data{phonebook}->[$num]->{memory} = $1;
 
 			} elsif ($key eq 'email' && $val =~ /"(\S+?)"/) {
 				$data{email} = $1;
@@ -269,11 +301,28 @@ sub _getQueryData {
 	my %subrMap = (
 			'set_destination' => 'divert_simple',
 			'get_destination' => 'getpin_simple',
-			'get_phonebook'   => 'getpin_post',
-			'get_email'       => 'getpin_post',
+			'get_phonebook'   => 'getpin_simple',
+			'get_email'       => 'getpin_simple',
+			'get_icd'         => 'getpin_simple',
 		);
 
 	my %queries = (
+		'account_post' => {
+			'method' => 'POST',
+			'url' => 'https://www.flextel.ltd.uk/cgi-bin/account.sh',
+			'referer' => 'Referer=https://www.flextel.ltd.uk/cgi-bin/passthru.sh?f=account&h=logon',
+			'data' => {
+				'mode'    => 'logon',
+				'cust_id' => '@@account@@',
+				'flextel' => '',
+				'start'   => '1',
+				'total'   => '9999',
+				'control' => '',
+				'acc_no'  => '@@account@@',
+				'pwd'     => '@@password@@',
+				'Logon'   => 'Logon',
+			},
+		},
 		'getpin_simple' => {
 			'method' => 'GET',
 			'url' => 'https://www.flextel.ltd.uk/cgi-bin/reroute.sh',
@@ -387,13 +436,14 @@ WWW::FleXtel - Manipulate FleXtel phone number redirection
  my $flextel = WWW::FleXtel->new(
          number   => "0701776655",
          pin      => "1234",
-         account  => "A99999",
-         password => "password",
+         account  => "A99999", # not required
+         password => "password", # not required
      );
  
  printf("Diverted to %s\n", $flextel->get_destination);
- $flextel->set_destination(destination => "01923001122");
- printf("Diverted to %s\n", $flextel->get_destination);
+ printf("Diverted to %s\n", $flextel->set_destination(
+         destination => "01923001122"
+     );
  
  print Dumper($flextel->get_phonebook);
 
@@ -409,8 +459,8 @@ number redirection webpage.
  my $flextel = WWW::FleXtel->new(
          number   => "0701776655",
          pin      => "1234",
-         account  => "A99999",
-         password => "password",
+         account  => "A99999", # not required
+         password => "password", # not required
      );
 
 Create a new WWW::FleXtel object. Currently the I<account> and
@@ -421,6 +471,39 @@ This method does have any mandatory parameters. However values passed
 this constructor method will be used as default fallback values if they
 are not passed to the subsequent accessor methods detailed below.
 
+=over 4
+
+=item number
+
+Specifies the default FleXtel number to use for all subsequent queries.
+
+=item pin
+
+Specifies the default PIN to use for all subsqeuent queries.
+
+=item account
+
+Specifies the default FleXtel account number to use for all subsequent
+queries. This parameter is not currently used, but may be used in future
+releases.
+
+=item password
+
+Specifies the default account password to use for all subsequent queries.
+This parameter is not currently use, but may be used in future releases.
+
+=item timeout
+
+Specifies (in seconds) the timeout for all HTTP connections. By default
+this is set to 15 seconds.
+
+=item cache_ttl
+
+Specifies (in seconds) the TTL for values to be cached internally within
+the WWW::FleXtel object. By default this is set to 5 seconds.
+
+=back
+
 =head2 get_destination
 
  my $destination = $flextel->get_destination;
@@ -446,26 +529,32 @@ diverted to.
  my $destination = $flextel->get_destination;
  my ($person) = grep(/\S/, map {
          $_->{title} if defined $_ && $_->{number} eq $destination
-     } @{$phonebook}) || "*not recorded*";
+     } @{$phonebook}); $person ||= "*not recorded*";
  print "$destination is $person in your phonebook\n";
 
 This method extracts the indexes, names and numbers from your FleXtel
-number's phonebook. This method has the potential to stop working in
-the future since it gathers the information by performing very basic
-parsing of the JavaScript object assignments on your FleXtel number's
-rerouting webpage.
+number's phonebook.
+
+=head2 get_icd
+
+ my $icd = $flextel->get_icd;
 
 =head2 get_email
 
  my $notification_address = $flextel->get_email;
 
+=head1 TODO
+
+Add support for retrieving a list of all FleXtel phone numbers
+attached to an account number.
+
 =head1 SEE ALSO
 
 L<http://www.flextel.ltd.uk>
 
 =head1 VERSION
 
-$Id: DMIDecode.pm 809 2006-10-22 12:47:45Z nicolaw $
+$Id: FleXtel.pm 942 2007-02-06 18:51:21Z nicolaw $
 
 =head1 AUTHOR
 
@@ -478,6 +567,14 @@ author something nice from her
 L<Amazon wishlist|http://www.amazon.co.uk/gp/registry/1VZXC59ESWYK0?sort=priority>? 
 ( http://www.amazon.co.uk/gp/registry/1VZXC59ESWYK0?sort=priority )
 
+=head1 ACKNOWLEDGEMENTS
+
+Special thanks to Kevin Archer at FleXtel and the FleXtel support and 
+development team for implementing the simple CVS access methods to their
+website.
+
+See CREDITS in the distribution tarball.
+
 =head1 COPYRIGHT
 
 Copyright 2007 Nicola Worthington.
